#!/usr/bin/env python
#
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegweb main views."""

from django.http import Http404
from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.shortcuts import redirect
from django.template import RequestContext
from django.views.decorators.http import require_POST
from django.contrib.auth.views import password_change as password_change_orig
from django.contrib.auth.views import password_change_done as password_change_done_orig

from pykeg.core import models
from pykeg.web.kegweb import forms
from pykeg.notification.forms import NotificationSettingsForm
from pykeg.notification.models import NotificationSettings

@login_required
def account_main(request):
    context = RequestContext(request)
    context['user'] = request.user
    return render_to_response('account/index.html', context_instance=context)

@login_required
def edit_mugshot(request):
    context = RequestContext(request)
    context['mugshot_form'] = forms.MugshotForm()
    if request.method == 'POST':
        form = forms.MugshotForm(request.POST, request.FILES)
        context['mugshot_form'] = form
        if form.is_valid():
            pic = models.Picture.objects.create()
            image = request.FILES['new_mugshot']
            pic.image.save(image.name, image)
            pic.save()

            profile = request.user.get_profile()
            profile.mugshot = pic
            profile.save()
    return render_to_response('account/mugshot.html', context_instance=context)

@login_required
def notifications(request):
    # TODO(mikey): Dynamically add settings forms for other e-mail
    # backends (currently hardcoded to email backend).
    context = RequestContext(request)
    existing_settings = NotificationSettings.objects.get_or_create(user=request.user,
        backend='pykeg.notification.backends.email.EmailNotificationBackend')[0]
    if request.method == 'POST':
        form = NotificationSettingsForm(request.POST, instance=existing_settings)
        if form.is_valid():
            instance = form.save(commit=False)
            instance.user = request.user
            instance.backend = 'pykeg.notification.backends.email.EmailNotificationBackend'
            instance.save()
            messages.success(request, 'Settings updated')
            existing_settings = instance
    context['form'] = NotificationSettingsForm(instance=existing_settings)
    return render_to_response('account/notifications.html', context_instance=context)

@login_required
@require_POST
def regenerate_api_key(request):
    form = forms.RegenerateApiKeyForm(request.POST)
    if form.is_valid():
        key, is_new = models.ApiKey.objects.get_or_create(user=request.user)
        key.regenerate()
        key.save()
    return redirect('kb-account-main')

def password_change(request, *args, **kwargs):
    kwargs['template_name'] = 'account/password_change.html'
    kwargs['post_change_redirect'] = reverse('password_change_done')
    return password_change_orig(request, *args, **kwargs)

def password_change_done(request):
    return password_change_done_orig(request, 'account/password_change_done.html')

@login_required
def plugin_settings(request, plugin_name):
    context = RequestContext(request)
    plugin = request.plugins.get(plugin_name, None)
    if not plugin:
        raise Http404('Plugin "%s" not loaded' % plugin_name)

    view = plugin.get_user_settings_view()
    if not view:
        raise Http404('No user settings for this plugin')

    return view(request, plugin)



# Cloned from django-extensions project: django_extensions.db.fields.json
# django_extensions has the following license:
#
#   Copyright (c) 2007 Michael Trier
#
#   Permission is hereby granted, free of charge, to any person obtaining a copy
#   of this software and associated documentation files (the "Software"), to
#   deal in the Software without restriction, including without limitation the
#   rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#   sell copies of the Software, and to permit persons to whom the Software is
#   furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in
#   all copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#   FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
#   IN THE SOFTWARE.
#
# Local differences from upstream version:
#   - disabled assert in JSONEncoder
#   - added south introspection rules
#   - use kbjson common json encode/decode methods

"""
JSONField automatically serializes most Python terms to JSON data.
Creates a TEXT field with a default value of "{}".  See test_json.py for
more information.

 from django.db import models
 from django_extensions.db.fields import json

 class LOL(models.Model):
     extra = json.JSONField()
"""

from kegbot.util import kbjson
from django.db import models

class JSONDict(dict):
    """
    Hack so repr() called by dumpdata will output JSON instead of
    Python formatted data.  This way fixtures will work!
    """
    def __repr__(self):
        return kbjson.dumps(self)

class JSONField(models.TextField):
    """JSONField is a generic textfield that neatly serializes/unserializes
    JSON objects seamlessly.  Main thingy must be a dict object."""

    # Used so to_python() is called
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = '{}'
        models.TextField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        """Convert our string value to JSON after we load it from the DB"""
        if not value:
            return {}
        elif isinstance(value, basestring):
            res = kbjson.loads(value)
            assert isinstance(res, dict)
            return JSONDict(**res)
        else:
            return value

    def get_db_prep_save(self, value, connection):
        """Convert our JSON object to a string before we save"""
        if not value:
            return super(JSONField, self).get_db_prep_save("", connection=connection)
        else:
            return super(JSONField, self).get_db_prep_save(kbjson.dumps(value),
                connection=connection)

try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([
      ([JSONField], [], {}),
      ], ["^pykeg\.core\.jsonfield\.JSONField"])



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Webhook plugin for Kegbot."""

from django.conf import settings
from pykeg.plugin import plugin
from pykeg.plugin import util

from . import forms
from . import tasks
from . import views

KEY_SITE_SETTINGS = 'settings'

class WebhookPlugin(plugin.Plugin):
    NAME = 'Web Hooks'
    SHORT_NAME = 'webhook'
    DESCRIPTION = 'Posts each system event to configured URLs'
    URL = 'http://kegbot.org'
    VERSION = '1.0.0'

    def get_admin_settings_view(self):
        return views.admin_settings

    def handle_new_event(self, event):
        self.logger.info('Handling new event: %s' % event.id)
        settings = self.get_site_settings()
        urls = settings.get('webhook_urls', '').strip().split()
        for url in urls:
            tasks.post_webhook.delay(url, event)

    ### Webook-specific methods

    def get_site_settings_form(self):
        form = forms.SiteSettingsForm()
        self.load_form_defaults(form, KEY_SITE_SETTINGS)
        return form

    def get_site_settings(self):
        return self.get_saved_form_data(forms.SiteSettingsForm(), KEY_SITE_SETTINGS)

    def save_site_settings_form(self, form):
        self.save_form(form, KEY_SITE_SETTINGS)




# Copyright 2014 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Notification-related models."""

from django.db import models

class NotificationSettings(models.Model):
    """Stores a user's notification settings for a specific backend."""

    class Meta:
        unique_together = ('user', 'backend')

    user = models.ForeignKey('auth.User',
        help_text='User for these settings.')
    backend = models.TextField(
        help_text='Notification backend (dotted path) for these settings.')
    keg_tapped = models.BooleanField(default=True,
        help_text='Sent when a keg is activated.')
    session_started = models.BooleanField(default=False,
        help_text='Sent when a new drinking session starts.')
    keg_volume_low = models.BooleanField(default=False,
        help_text='Sent when a keg becomes low.')
    keg_ended = models.BooleanField(default=False,
        help_text='Sent when a keg has been taken offline.')
